@@ -130,10 +130,10 @@ static GtkActionEntry rb_playlist_manager_actions [] =
 	/* Submenu of Music */
 	{ "Playlist", NULL, N_("_Playlist") },
 
-	{ "MusicPlaylistNewPlaylist", GNOME_MEDIA_PLAYLIST, N_("_New Playlist"), "<control>N",
+	{ "MusicPlaylistNewPlaylist", RB_STOCK_PLAYLIST, N_("_New Playlist"), "<control>N",
 	  N_("Create a new playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_new_playlist) },
-	{ "MusicPlaylistNewAutomaticPlaylist", GNOME_MEDIA_AUTO_PLAYLIST, N_("New _Automatic Playlist..."), NULL,
+	{ "MusicPlaylistNewAutomaticPlaylist", RB_STOCK_AUTO_PLAYLIST, N_("New _Automatic Playlist..."), NULL,
 	  N_("Create a new automatically updating playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_new_automatic_playlist) },
 	{ "MusicPlaylistLoadPlaylist", NULL, N_("_Load from File..."), NULL,
@@ -502,14 +502,28 @@ rb_playlist_manager_error_quark (void)
 	return quark;
 }
 
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
+static void
+handle_playlist_entry_cb (TotemPlParser *playlist,
+			  const char *uri_maybe,
+			  GHashTable *metadata,
+			  RBPlaylistManager *mgr)
+#else
 static void
 handle_playlist_entry_cb (TotemPlParser *playlist,
 			  const char *uri_maybe,
 			  const char *title,
 			  const char *genre,
 			  RBPlaylistManager *mgr)
+#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 {
 	char *uri;
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
+	const char *title, *genre;
+
+	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
+	genre = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_GENRE);
+#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 
 	uri = rb_canonicalise_uri (uri_maybe);
 	g_return_if_fail (uri != NULL);
@@ -577,9 +591,15 @@ rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri, GError
 	{
 		TotemPlParser *parser = totem_pl_parser_new ();
 
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
+		g_signal_connect_object (parser, "entry-parsed",
+					 G_CALLBACK (handle_playlist_entry_cb),
+					 mgr, 0);
+#else
 		g_signal_connect_object (parser, "entry",
 					 G_CALLBACK (handle_playlist_entry_cb),
 					 mgr, 0);
+#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 
 		g_signal_connect_object (parser, "playlist-start",
 					 G_CALLBACK (playlist_load_start_cb),
@@ -788,6 +808,38 @@ rb_playlist_manager_save_data (struct RBPlaylistManagerSaveData *data)
 	return NULL;
 }
 
+static gboolean
+model_foreach_func (GtkTreeModel *model,
+		    GtkTreePath  *path,
+		    GtkTreeIter  *iter,
+		    xmlNodePtr    root)
+{
+	RBSource *source;
+	gboolean  local;
+
+	gtk_tree_model_get (model,
+			    iter,
+			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source,
+			    -1);
+	if (source == NULL) {
+		goto out;
+	}
+	if (RB_IS_PLAYLIST_SOURCE (source) == FALSE) {
+		goto out;
+	}
+
+	g_object_get (source, "is-local", &local, NULL);
+	if (local) {
+		rb_playlist_source_save_to_xml (RB_PLAYLIST_SOURCE (source), root);
+	}
+ out:
+	if (source != NULL) {
+		g_object_unref (source);
+	}
+
+	return FALSE;
+}
+
 /**
  * rb_playlist_manager_save_playlists
  * @mgr: the #RBPlaylistManager
@@ -806,7 +858,6 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
 {
 	xmlNodePtr root;
 	struct RBPlaylistManagerSaveData *data;
-	GtkTreeIter iter;
 	GtkTreeModel *fmodel;
 	GtkTreeModel *model;
 
@@ -832,31 +883,7 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
 	g_object_unref (fmodel);
 
-	if (gtk_tree_model_get_iter_first (model, &iter)) {
-		do {
-			RBSource *source;
-			gboolean local;
-
-			gtk_tree_model_get (model,
-					    &iter,
-					    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-					    &source,
-					    -1);
-			if (source == NULL) {
-				continue;
-			}
-			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE) {
-				g_object_unref (source);
-				continue;
-			}
-
-			g_object_get (source, "is-local", &local, NULL);
-			if (local)
-				rb_playlist_source_save_to_xml (RB_PLAYLIST_SOURCE (source), root);
-
-			g_object_unref (source);
-		} while (gtk_tree_model_iter_next (model, &iter));
-	}
+	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc)model_foreach_func, root);
 
 	/* mark clean here.  if the save fails, we'll mark it dirty again */
 	rb_playlist_manager_set_dirty (data->mgr, FALSE);
