@@ -42,10 +42,9 @@
 #include "rb-property-view.h"
 #include "rb-util.h"
 #include "rb-file-helpers.h"
-#include "totem-pl-parser.h"
 #include "rb-preferences.h"
 #include "rb-dialog.h"
-#include "rb-new-podcast-dialog.h"
+#include "rb-uri-dialog.h"
 #include "rb-podcast-properties-dialog.h"
 #include "rb-feed-podcast-properties-dialog.h"
 #include "rb-playlist-manager.h"
@@ -191,7 +190,7 @@ static void rb_podcast_source_finish_download_cb 	(RBPodcastManager *pd,
 							 RhythmDBEntry *entry,
 							 RBPodcastSource *source);
 
-static void rb_podcast_source_feed_updates_avaliable_cb (RBPodcastManager *pd,
+static void rb_podcast_source_feed_updates_available_cb (RBPodcastManager *pd,
 							 RhythmDBEntry *entry,
 							 RBPodcastSource *source);
 
@@ -380,7 +379,6 @@ static void
 rb_podcast_source_init (RBPodcastSource *source)
 {
 	GdkPixbuf *pixbuf;
-	GdkPixbuf *scaled = NULL;
 	gint       size;
 
 	source->priv = RB_PODCAST_SOURCE_GET_PRIVATE (source);
@@ -391,29 +389,16 @@ rb_podcast_source_init (RBPodcastSource *source)
 
 	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
 
-	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
 	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
 					   RB_STOCK_PODCAST,
 					   size,
 					   0, NULL);
 
 	if (pixbuf != NULL) {
-		/* we only have this icon as a 48x48 bitmap, so it helps to
-		 * scale it to the right size.  otherwise the sourcelist gets
-		 * a bit uneven.
-		 */
-		gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
-		scaled = gdk_pixbuf_scale_simple (pixbuf, size, size, GDK_INTERP_BILINEAR);
-
-		rb_source_set_pixbuf (RB_SOURCE (source), scaled);
-	}
-
-	if (pixbuf != NULL) {
+		rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
 		g_object_unref (pixbuf);
 	}
-	if (scaled != NULL) {
-		g_object_unref (scaled);
-	}
 }
 
 static void
@@ -433,8 +418,26 @@ rb_podcast_source_dispose (GObject *object)
 
 	if (source->priv->db != NULL) {
 		g_object_unref (source->priv->db);
+		source->priv->db = NULL;
 	}
 
+	if (source->priv->cached_all_query) {
+		g_object_unref (source->priv->cached_all_query);
+		source->priv->cached_all_query = NULL;
+	}
+
+	if (source->priv->action_group != NULL) {
+		g_object_unref (source->priv->action_group);
+		source->priv->action_group = NULL;
+	}
+
+	if (source->priv->podcast_mgr != NULL) {
+		g_object_unref (source->priv->podcast_mgr);
+		source->priv->podcast_mgr = NULL;
+	}
+
+	eel_gconf_notification_remove (source->priv->prefs_notify_id);
+
 	G_OBJECT_CLASS (rb_podcast_source_parent_class)->dispose (object);
 }
 
@@ -452,22 +455,11 @@ rb_podcast_source_finalize (GObject *object)
 
 	rb_debug ("finalizing podcast source");
 
-	if (source->priv->action_group != NULL) {
-		g_object_unref (source->priv->action_group);
-	}
-
-	eel_gconf_notification_remove (source->priv->prefs_notify_id);
-
-	g_object_unref (source->priv->podcast_mgr);
-
 	if (source->priv->selected_feeds) {
 		g_list_foreach (source->priv->selected_feeds, (GFunc) g_free, NULL);
 	        g_list_free (source->priv->selected_feeds);
 	}
 
-       if (source->priv->cached_all_query)
-		g_object_unref (G_OBJECT (source->priv->cached_all_query));
-
 	G_OBJECT_CLASS (rb_podcast_source_parent_class)->finalize (object);
 }
 
@@ -593,8 +585,8 @@ rb_podcast_source_constructor (GType type,
 						 source, NULL);
 
 	rb_entry_view_append_column_custom (source->priv->posts, column,
-					    _("_Date"), "Date",
-					    (GCompareDataFunc) rb_podcast_source_post_date_cell_sort_func, 0);
+					    _("Date"), "Date",
+					    (GCompareDataFunc) rb_podcast_source_post_date_cell_sort_func, 0, NULL);
 
 	rb_entry_view_append_column (source->priv->posts, RB_ENTRY_VIEW_COL_TITLE, TRUE);
 
@@ -614,8 +606,8 @@ rb_podcast_source_constructor (GType type,
 						 source, NULL);
 
 	rb_entry_view_append_column_custom (source->priv->posts, column,
-					    _("_Feed"), "Feed",
-					    (GCompareDataFunc) rb_podcast_source_post_feed_cell_sort_func, 0);
+					    _("Feed"), "Feed",
+					    (GCompareDataFunc) rb_podcast_source_post_feed_cell_sort_func, 0, NULL);
 
 	rb_entry_view_append_column (source->priv->posts, RB_ENTRY_VIEW_COL_DURATION, FALSE);
 	rb_entry_view_append_column (source->priv->posts, RB_ENTRY_VIEW_COL_RATING, FALSE);
@@ -652,7 +644,7 @@ rb_podcast_source_constructor (GType type,
 
 	rb_entry_view_append_column_custom (source->priv->posts, column,
 					    _("Status"), "Status",
-					    (GCompareDataFunc) rb_podcast_source_post_status_cell_sort_func, 0);
+					    (GCompareDataFunc) rb_podcast_source_post_status_cell_sort_func, 0, NULL);
 
 	g_signal_connect_object (G_OBJECT (source->priv->posts),
 				 "sort-order-changed",
@@ -779,6 +771,8 @@ rb_podcast_source_set_property (GObject *object,
 	switch (prop_id) {
 	case PROP_PODCAST_MANAGER:
 		source->priv->podcast_mgr = g_value_get_object (value);
+		if (source->priv->podcast_mgr)
+			g_object_ref (source->priv->podcast_mgr);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -812,6 +806,7 @@ rb_podcast_source_new (RBShell *shell)
 					  "name", _("Podcasts"),
 					  "shell", shell,
 					  "entry-type", RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
+					  "source-group", RB_SOURCE_GROUP_LIBRARY,
 					  NULL));
 
 	rb_shell_register_entry_type_for_source (shell, source,
@@ -1236,6 +1231,9 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 	source->priv->config_widget = glade_xml_get_widget (xml, "podcast_vbox");
 
 	btn_file = glade_xml_get_widget (xml, "location_chooser");
+	gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (btn_file),
+					      rb_music_dir (),
+					      NULL);
 	download_dir = rb_podcast_manager_get_podcast_dir (source->priv->podcast_mgr);
 	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (btn_file),
 						 download_dir);
@@ -1327,7 +1325,6 @@ rb_podcast_source_cmd_cancel_download (GtkAction *action,
 	GValue val = {0, };
 	RBEntryView *posts;
 
-	rb_debug ("Add to download action");
 	posts = source->priv->posts;
 
 	lst = rb_entry_view_get_selected_entries (posts);
@@ -1600,7 +1597,7 @@ rb_podcast_source_feed_title_cell_data_func (GtkTreeViewColumn *column,
 		/* Subtract one for the All node */
 		nodes--;
 
-		fmt = ngettext ("All %d feed", "All %d feeds", nodes);
+		fmt = ngettext ("%d feed", "All %d feeds", nodes);
 
 		str = g_strdup_printf (fmt, nodes, number);
 	} else {
@@ -1632,11 +1629,7 @@ rb_podcast_source_post_date_cell_data_func (GtkTreeViewColumn *column,
         if (value == 0) {
 		str = g_strdup (_("Unknown"));
 	} else {
-		struct tm time_tm;
-		time_t time = (time_t) value;
-
-		localtime_r (&time, &time_tm);
-		str = eel_strdup_strftime (_("%Y-%m-%d %H:%M"), &time_tm);
+		str = rb_utf_friendly_time (value);
 	}
 
 	g_object_set (G_OBJECT (renderer), "text", str, NULL);
@@ -1673,8 +1666,8 @@ rb_podcast_source_load_finish_cb  (gpointer cb_data)
 			  	source);
 
 	g_signal_connect_after (G_OBJECT (source->priv->podcast_mgr),
-			  	"feed_updates_avaliable",
- 			  	G_CALLBACK (rb_podcast_source_feed_updates_avaliable_cb),
+			  	"feed_updates_available",
+ 			  	G_CALLBACK (rb_podcast_source_feed_updates_available_cb),
 			  	source);
 
 	return FALSE;
@@ -1742,7 +1735,7 @@ rb_podcast_source_start_download_cb (RBPodcastManager *pd,
 	const gchar *podcast_name = rhythmdb_entry_get_string(entry, RHYTHMDB_PROP_TITLE);
 
 	rb_debug ("Start download");
-	rb_shell_hidden_notify (shell, 4000, _("Downloading podcast"), NULL, podcast_name);
+	rb_shell_hidden_notify (shell, 4000, _("Downloading podcast"), NULL, podcast_name, FALSE);
 	g_object_unref (shell);
 }
 
@@ -1755,20 +1748,20 @@ rb_podcast_source_finish_download_cb (RBPodcastManager *pd,
 	const gchar *podcast_name = rhythmdb_entry_get_string(entry, RHYTHMDB_PROP_TITLE);
 
 	rb_debug ("Finish download");
-	rb_shell_hidden_notify (shell, 4000, _("Finished downloading podcast"), NULL, podcast_name);
+	rb_shell_hidden_notify (shell, 4000, _("Finished downloading podcast"), NULL, podcast_name, FALSE);
 	g_object_unref (shell);
 }
 
 static void
-rb_podcast_source_feed_updates_avaliable_cb (RBPodcastManager *pd,
+rb_podcast_source_feed_updates_available_cb (RBPodcastManager *pd,
 					     RhythmDBEntry *entry,
 					     RBPodcastSource *source)
 {
 	RBShell *shell = rb_podcast_source_get_shell (source);
 	const gchar *podcast_name = rhythmdb_entry_get_string(entry, RHYTHMDB_PROP_TITLE);
 
-	rb_debug ("Updates avaliable");
-	rb_shell_hidden_notify (shell, 4000, _("New updates avaliable from"), NULL, podcast_name);
+	rb_debug ("Updates available");
+	rb_shell_hidden_notify (shell, 4000, _("New updates available from"), NULL, podcast_name, FALSE);
 	g_object_unref (shell);
 
 }
@@ -1787,7 +1780,7 @@ void
 rb_podcast_source_shutdown	(RBPodcastSource *source)
 {
 	rb_debug ("podcast source shutdown");
-	rb_podcast_manager_cancel_all (source->priv->podcast_mgr);
+	rb_podcast_manager_shutdown (source->priv->podcast_mgr);
 }
 
 static gint
@@ -1970,21 +1963,29 @@ impl_get_search_actions (RBSource *source)
 	return actions;
 }
 
+static void
+rb_podcast_source_location_added_cb (RBURIDialog *dialog,
+				     const char *location,
+				     RBPodcastSource *source)
+{
+	rb_podcast_manager_subscribe_feed (source->priv->podcast_mgr, location);
+}
+
 static void
 rb_podcast_source_cmd_new_podcast (GtkAction *action,
 				   RBPodcastSource *source)
 {
 	GtkWidget *dialog;
-	GObject *object;
 
 	rb_debug ("Got new podcast command");
-	g_object_get (source, "podcast-manager", &object, NULL);
 
-	dialog = rb_new_podcast_dialog_new (RB_PODCAST_MANAGER (object));
+	dialog = rb_uri_dialog_new (_("New Podcast Feed"), _("URL of podcast feed:"));
+	g_signal_connect_object (G_OBJECT (dialog),
+				 "location-added",
+				 G_CALLBACK (rb_podcast_source_location_added_cb),
+				 source, 0);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
-
-	g_object_unref (object);
 }
 
 static void
