@@ -26,6 +26,7 @@
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkactiongroup.h>
 
+#include "rb-source-group.h"
 #include "rb-entry-view.h"
 #include "rb-shell-preferences.h"
 
@@ -38,22 +39,9 @@ typedef enum {
 	RB_SOURCE_EOF_NEXT,
 } RBSourceEOFType;
 
-/* groupings in the source list */
-typedef enum {
-	RB_SOURCELIST_GROUP_FIXED,	/* library, iradio, podcast, queue */
-	RB_SOURCELIST_GROUP_PERSISTANT,	/* playlists */
-	RB_SOURCELIST_GROUP_REMOVABLE,	/* ipod, generic audio player, audio CD */
-	RB_SOURCELIST_GROUP_TRANSIENT,	/* DAAP shares */
-
-	RB_SOURCELIST_GROUP_LAST
-} RBSourceListGroup;
-
 GType rb_source_eof_type_get_type (void);
 #define RB_TYPE_SOURCE_EOF_TYPE	(rb_source_eof_type_get_type())
 
-GType rb_sourcelist_group_get_type (void);
-#define RB_TYPE_SOURCELIST_GROUP (rb_sourcelist_group_get_type())
-
 #define RB_TYPE_SOURCE         (rb_source_get_type ())
 #define RB_SOURCE(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), RB_TYPE_SOURCE, RBSource))
 #define RB_SOURCE_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), RB_TYPE_SOURCE, RBSourceClass))
@@ -61,6 +49,8 @@ GType rb_sourcelist_group_get_type (void);
 #define RB_IS_SOURCE_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_SOURCE))
 #define RB_SOURCE_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_SOURCE, RBSourceClass))
 
+#define RB_SOURCE_ICON_SIZE	GTK_ICON_SIZE_LARGE_TOOLBAR
+
 typedef struct _RBSourceClass	RBSourceClass;
 typedef struct _RBSource		RBSource;
 typedef struct _RBSourcePrivate	RBSourcePrivate;
@@ -172,6 +162,7 @@ gboolean	rb_source_can_move_to_trash	(RBSource *source);
 gboolean	rb_source_can_copy		(RBSource *source);
 gboolean	rb_source_can_paste		(RBSource *source);
 gboolean	rb_source_can_add_to_queue	(RBSource *source);
+gboolean	rb_source_can_show_properties	(RBSource *source);
 
 GList *		rb_source_cut			(RBSource *source);
 GList *		rb_source_copy			(RBSource *source);
